package org.joda.time.tz;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.util.Map.Entry;
import org.joda.time.Chronology;
import org.joda.time.DateTime;
import org.joda.time.DateTimeField;
import org.joda.time.DateTimeZone;
import org.joda.time.LocalDate;
import org.joda.time.MutableDateTime;
import org.joda.time.chrono.ISOChronology;
import org.joda.time.chrono.LenientChronology;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;
import org.joda.time.tz.DateTimeZoneBuilder;
import org.joda.time.tz.ZoneInfoLogger;

public class ZoneInfoCompiler {

   static ZoneInfoCompiler.DateTimeOfYear cStartOfYear;
   static Chronology cLenientISO;
   private Map iRuleSets = new HashMap();
   private List iZones = new ArrayList();
   private List iGoodLinks = new ArrayList();
   private List iBackLinks = new ArrayList();


   public static void main(String[] var0) throws Exception {
      if(var0.length == 0) {
         printUsage();
      } else {
         File var1 = null;
         File var2 = null;
         boolean var3 = false;

         int var4;
         for(var4 = 0; var4 < var0.length; ++var4) {
            try {
               if("-src".equals(var0[var4])) {
                  ++var4;
                  var1 = new File(var0[var4]);
               } else if("-dst".equals(var0[var4])) {
                  ++var4;
                  var2 = new File(var0[var4]);
               } else {
                  if(!"-verbose".equals(var0[var4])) {
                     if("-?".equals(var0[var4])) {
                        printUsage();
                        return;
                     }
                     break;
                  }

                  var3 = true;
               }
            } catch (IndexOutOfBoundsException var7) {
               printUsage();
               return;
            }
         }

         if(var4 >= var0.length) {
            printUsage();
         } else {
            File[] var5 = new File[var0.length - var4];

            for(int var6 = 0; var4 < var0.length; ++var6) {
               var5[var6] = var1 == null?new File(var0[var4]):new File(var1, var0[var4]);
               ++var4;
            }

            ZoneInfoLogger.set(var3);
            ZoneInfoCompiler var8 = new ZoneInfoCompiler();
            var8.compile(var2, var5);
         }
      }
   }

   private static void printUsage() {
      System.out.println("Usage: java org.joda.time.tz.ZoneInfoCompiler <options> <source files>");
      System.out.println("where possible options include:");
      System.out.println("  -src <directory>    Specify where to read source files");
      System.out.println("  -dst <directory>    Specify where to write generated files");
      System.out.println("  -verbose            Output verbosely (default false)");
   }

   static ZoneInfoCompiler.DateTimeOfYear getStartOfYear() {
      if(cStartOfYear == null) {
         cStartOfYear = new ZoneInfoCompiler.DateTimeOfYear();
      }

      return cStartOfYear;
   }

   static Chronology getLenientISOChronology() {
      if(cLenientISO == null) {
         cLenientISO = LenientChronology.getInstance(ISOChronology.getInstanceUTC());
      }

      return cLenientISO;
   }

   static void writeZoneInfoMap(DataOutputStream var0, Map var1) throws IOException {
      HashMap var2 = new HashMap(var1.size());
      TreeMap var3 = new TreeMap();
      short var4 = 0;
      Iterator var5 = var1.entrySet().iterator();

      Entry var6;
      String var7;
      while(var5.hasNext()) {
         var6 = (Entry)var5.next();
         var7 = (String)var6.getKey();
         Short var8;
         if(!var2.containsKey(var7)) {
            var8 = Short.valueOf(var4);
            var2.put(var7, var8);
            var3.put(var8, var7);
            ++var4;
            if(var4 == 0) {
               throw new InternalError("Too many time zone ids");
            }
         }

         var7 = ((DateTimeZone)var6.getValue()).getID();
         if(!var2.containsKey(var7)) {
            var8 = Short.valueOf(var4);
            var2.put(var7, var8);
            var3.put(var8, var7);
            ++var4;
            if(var4 == 0) {
               throw new InternalError("Too many time zone ids");
            }
         }
      }

      var0.writeShort(var3.size());
      var5 = var3.values().iterator();

      while(var5.hasNext()) {
         String var9 = (String)var5.next();
         var0.writeUTF(var9);
      }

      var0.writeShort(var1.size());
      var5 = var1.entrySet().iterator();

      while(var5.hasNext()) {
         var6 = (Entry)var5.next();
         var7 = (String)var6.getKey();
         var0.writeShort(((Short)var2.get(var7)).shortValue());
         var7 = ((DateTimeZone)var6.getValue()).getID();
         var0.writeShort(((Short)var2.get(var7)).shortValue());
      }

   }

   static int parseYear(String var0, int var1) {
      var0 = var0.toLowerCase();
      return !var0.equals("minimum") && !var0.equals("min")?(!var0.equals("maximum") && !var0.equals("max")?(var0.equals("only")?var1:Integer.parseInt(var0)):Integer.MAX_VALUE):Integer.MIN_VALUE;
   }

   static int parseMonth(String var0) {
      DateTimeField var1 = ISOChronology.getInstanceUTC().monthOfYear();
      return var1.get(var1.set(0L, var0, Locale.ENGLISH));
   }

   static int parseDayOfWeek(String var0) {
      DateTimeField var1 = ISOChronology.getInstanceUTC().dayOfWeek();
      return var1.get(var1.set(0L, var0, Locale.ENGLISH));
   }

   static String parseOptional(String var0) {
      return var0.equals("-")?null:var0;
   }

   static int parseTime(String var0) {
      DateTimeFormatter var1 = ISODateTimeFormat.hourMinuteSecondFraction();
      MutableDateTime var2 = new MutableDateTime(0L, getLenientISOChronology());
      byte var3 = 0;
      if(var0.startsWith("-")) {
         var3 = 1;
      }

      int var4 = var1.parseInto(var2, var0, var3);
      if(var4 == ~var3) {
         throw new IllegalArgumentException(var0);
      } else {
         int var5 = (int)var2.getMillis();
         if(var3 == 1) {
            var5 = -var5;
         }

         return var5;
      }
   }

   static char parseZoneChar(char var0) {
      switch(var0) {
      case 71:
      case 85:
      case 90:
      case 103:
      case 117:
      case 122:
         return 'u';
      case 83:
      case 115:
         return 's';
      case 87:
      case 119:
      default:
         return 'w';
      }
   }

   static boolean test(String var0, DateTimeZone var1) {
      if(!var0.equals(var1.getID())) {
         return true;
      } else {
         long var2 = ISOChronology.getInstanceUTC().year().set(0L, 1850);
         long var4 = ISOChronology.getInstanceUTC().year().set(0L, 2050);
         int var6 = var1.getOffset(var2);
         String var7 = var1.getNameKey(var2);
         ArrayList var8 = new ArrayList();

         while(true) {
            long var9 = var1.nextTransition(var2);
            if(var9 != var2 && var9 <= var4) {
               var2 = var9;
               int var11 = var1.getOffset(var9);
               String var15 = var1.getNameKey(var9);
               if(var6 == var11 && var7.equals(var15)) {
                  System.out.println("*d* Error in " + var1.getID() + " " + new DateTime(var9, ISOChronology.getInstanceUTC()));
                  return false;
               }

               if(var15 != null && (var15.length() >= 3 || "??".equals(var15))) {
                  var8.add(Long.valueOf(var9));
                  var6 = var11;
                  var7 = var15;
                  continue;
               }

               System.out.println("*s* Error in " + var1.getID() + " " + new DateTime(var9, ISOChronology.getInstanceUTC()) + ", nameKey=" + var15);
               return false;
            }

            var2 = ISOChronology.getInstanceUTC().year().set(0L, 2050);
            var4 = ISOChronology.getInstanceUTC().year().set(0L, 1850);
            int var14 = var8.size();

            while(true) {
               --var14;
               if(var14 < 0) {
                  break;
               }

               long var10 = var1.previousTransition(var2);
               if(var10 == var2 || var10 < var4) {
                  break;
               }

               var2 = var10;
               long var12 = ((Long)var8.get(var14)).longValue();
               if(var12 - 1L != var10) {
                  System.out.println("*r* Error in " + var1.getID() + " " + new DateTime(var10, ISOChronology.getInstanceUTC()) + " != " + new DateTime(var12 - 1L, ISOChronology.getInstanceUTC()));
                  return false;
               }
            }

            return true;
         }
      }
   }

   public Map compile(File var1, File[] var2) throws IOException {
      if(var2 != null) {
         for(int var3 = 0; var3 < var2.length; ++var3) {
            BufferedReader var4 = new BufferedReader(new FileReader(var2[var3]));
            this.parseDataFile(var4, "backward".equals(var2[var3].getName()));
            var4.close();
         }
      }

      if(var1 != null) {
         if(!var1.exists() && !var1.mkdirs()) {
            throw new IOException("Destination directory doesn\'t exist and cannot be created: " + var1);
         }

         if(!var1.isDirectory()) {
            throw new IOException("Destination is not a directory: " + var1);
         }
      }

      TreeMap var14 = new TreeMap();
      TreeMap var15 = new TreeMap();
      System.out.println("Writing zoneinfo files");

      int var5;
      for(var5 = 0; var5 < this.iZones.size(); ++var5) {
         ZoneInfoCompiler.Zone var6 = (ZoneInfoCompiler.Zone)this.iZones.get(var5);
         DateTimeZoneBuilder var7 = new DateTimeZoneBuilder();
         var6.addToBuilder(var7, this.iRuleSets);
         DateTimeZone var8 = var7.toDateTimeZone(var6.iName, true);
         if(test(var8.getID(), var8)) {
            var14.put(var8.getID(), var8);
            var15.put(var8.getID(), var6);
            if(var1 != null) {
               this.writeZone(var1, var7, var8);
            }
         }
      }

      String var19;
      for(var5 = 0; var5 < this.iGoodLinks.size(); var5 += 2) {
         String var16 = (String)this.iGoodLinks.get(var5);
         var19 = (String)this.iGoodLinks.get(var5 + 1);
         ZoneInfoCompiler.Zone var22 = (ZoneInfoCompiler.Zone)var15.get(var16);
         if(var22 == null) {
            System.out.println("Cannot find source zone \'" + var16 + "\' to link alias \'" + var19 + "\' to");
         } else {
            DateTimeZoneBuilder var9 = new DateTimeZoneBuilder();
            var22.addToBuilder(var9, this.iRuleSets);
            DateTimeZone var10 = var9.toDateTimeZone(var19, true);
            if(test(var10.getID(), var10)) {
               var14.put(var10.getID(), var10);
               if(var1 != null) {
                  this.writeZone(var1, var9, var10);
               }
            }

            var14.put(var10.getID(), var10);
            if(ZoneInfoLogger.verbose()) {
               System.out.println("Good link: " + var19 + " -> " + var16 + " revived");
            }
         }
      }

      for(var5 = 0; var5 < 2; ++var5) {
         for(int var17 = 0; var17 < this.iBackLinks.size(); var17 += 2) {
            var19 = (String)this.iBackLinks.get(var17);
            String var23 = (String)this.iBackLinks.get(var17 + 1);
            DateTimeZone var25 = (DateTimeZone)var14.get(var19);
            if(var25 == null) {
               if(var5 > 0) {
                  System.out.println("Cannot find time zone \'" + var19 + "\' to link alias \'" + var23 + "\' to");
               }
            } else {
               var14.put(var23, var25);
               if(ZoneInfoLogger.verbose()) {
                  System.out.println("Back link: " + var23 + " -> " + var25.getID());
               }
            }
         }
      }

      if(var1 != null) {
         System.out.println("Writing ZoneInfoMap");
         File var18 = new File(var1, "ZoneInfoMap");
         if(!var18.getParentFile().exists()) {
            var18.getParentFile().mkdirs();
         }

         FileOutputStream var20 = new FileOutputStream(var18);
         DataOutputStream var21 = new DataOutputStream(var20);

         try {
            TreeMap var24 = new TreeMap(String.CASE_INSENSITIVE_ORDER);
            var24.putAll(var14);
            writeZoneInfoMap(var21, var24);
         } finally {
            var21.close();
         }
      }

      return var14;
   }

   private void writeZone(File var1, DateTimeZoneBuilder var2, DateTimeZone var3) throws IOException {
      if(ZoneInfoLogger.verbose()) {
         System.out.println("Writing " + var3.getID());
      }

      File var4 = new File(var1, var3.getID());
      if(!var4.getParentFile().exists()) {
         var4.getParentFile().mkdirs();
      }

      FileOutputStream var5 = new FileOutputStream(var4);

      try {
         var2.writeTo(var3.getID(), (OutputStream)var5);
      } finally {
         var5.close();
      }

      FileInputStream var6 = new FileInputStream(var4);
      DateTimeZone var7 = DateTimeZoneBuilder.readFrom((InputStream)var6, var3.getID());
      var6.close();
      if(!var3.equals(var7)) {
         System.out.println("*e* Error in " + var3.getID() + ": Didn\'t read properly from file");
      }

   }

   public void parseDataFile(BufferedReader var1, boolean var2) throws IOException {
      ZoneInfoCompiler.Zone var3 = null;

      String var4;
      while((var4 = var1.readLine()) != null) {
         String var5 = var4.trim();
         if(var5.length() != 0 && var5.charAt(0) != 35) {
            int var6 = var4.indexOf(35);
            if(var6 >= 0) {
               var4 = var4.substring(0, var6);
            }

            StringTokenizer var7 = new StringTokenizer(var4, " \t");
            if(Character.isWhitespace(var4.charAt(0)) && var7.hasMoreTokens()) {
               if(var3 != null) {
                  var3.chain(var7);
               }
            } else {
               if(var3 != null) {
                  this.iZones.add(var3);
               }

               var3 = null;
               if(var7.hasMoreTokens()) {
                  String var8 = var7.nextToken();
                  if(var8.equalsIgnoreCase("Rule")) {
                     ZoneInfoCompiler.Rule var11 = new ZoneInfoCompiler.Rule(var7);
                     ZoneInfoCompiler.RuleSet var12 = (ZoneInfoCompiler.RuleSet)this.iRuleSets.get(var11.iName);
                     if(var12 == null) {
                        var12 = new ZoneInfoCompiler.RuleSet(var11);
                        this.iRuleSets.put(var11.iName, var12);
                     } else {
                        var12.addRule(var11);
                     }
                  } else if(var8.equalsIgnoreCase("Zone")) {
                     var3 = new ZoneInfoCompiler.Zone(var7);
                  } else if(var8.equalsIgnoreCase("Link")) {
                     String var9 = var7.nextToken();
                     String var10 = var7.nextToken();
                     if(!var2 && !var10.equals("US/Pacific-New") && !var10.startsWith("Etc/") && !var10.equals("GMT")) {
                        this.iGoodLinks.add(var9);
                        this.iGoodLinks.add(var10);
                     } else {
                        this.iBackLinks.add(var9);
                        this.iBackLinks.add(var10);
                     }
                  } else {
                     System.out.println("Unknown line: " + var4);
                  }
               }
            }
         }
      }

      if(var3 != null) {
         this.iZones.add(var3);
      }

   }

   private static class Zone {

      public final String iName;
      public final int iOffsetMillis;
      public final String iRules;
      public final String iFormat;
      public final int iUntilYear;
      public final ZoneInfoCompiler.DateTimeOfYear iUntilDateTimeOfYear;
      private ZoneInfoCompiler.Zone iNext;


      Zone(StringTokenizer var1) {
         this(var1.nextToken(), var1);
      }

      private Zone(String var1, StringTokenizer var2) {
         this.iName = var1.intern();
         this.iOffsetMillis = ZoneInfoCompiler.parseTime(var2.nextToken());
         this.iRules = ZoneInfoCompiler.parseOptional(var2.nextToken());
         this.iFormat = var2.nextToken().intern();
         int var3 = Integer.MAX_VALUE;
         ZoneInfoCompiler.DateTimeOfYear var4 = ZoneInfoCompiler.getStartOfYear();
         if(var2.hasMoreTokens()) {
            var3 = Integer.parseInt(var2.nextToken());
            if(var2.hasMoreTokens()) {
               var4 = new ZoneInfoCompiler.DateTimeOfYear(var2);
            }
         }

         this.iUntilYear = var3;
         this.iUntilDateTimeOfYear = var4;
      }

      void chain(StringTokenizer var1) {
         if(this.iNext != null) {
            this.iNext.chain(var1);
         } else {
            this.iNext = new ZoneInfoCompiler.Zone(this.iName, var1);
         }

      }

      public void addToBuilder(DateTimeZoneBuilder var1, Map var2) {
         addToBuilder(this, var1, var2);
      }

      private static void addToBuilder(ZoneInfoCompiler.Zone var0, DateTimeZoneBuilder var1, Map var2) {
         while(true) {
            if(var0 != null) {
               var1.setStandardOffset(var0.iOffsetMillis);
               if(var0.iRules == null) {
                  var1.setFixedSavings(var0.iFormat, 0);
               } else {
                  try {
                     int var3 = ZoneInfoCompiler.parseTime(var0.iRules);
                     var1.setFixedSavings(var0.iFormat, var3);
                  } catch (Exception var5) {
                     ZoneInfoCompiler.RuleSet var4 = (ZoneInfoCompiler.RuleSet)var2.get(var0.iRules);
                     if(var4 == null) {
                        throw new IllegalArgumentException("Rules not found: " + var0.iRules);
                     }

                     var4.addRecurring(var1, var0.iFormat);
                  }
               }

               if(var0.iUntilYear != Integer.MAX_VALUE) {
                  var0.iUntilDateTimeOfYear.addCutover(var1, var0.iUntilYear);
                  var0 = var0.iNext;
                  continue;
               }
            }

            return;
         }
      }

      public String toString() {
         String var1 = "[Zone]\nName: " + this.iName + "\n" + "OffsetMillis: " + this.iOffsetMillis + "\n" + "Rules: " + this.iRules + "\n" + "Format: " + this.iFormat + "\n" + "UntilYear: " + this.iUntilYear + "\n" + this.iUntilDateTimeOfYear;
         return this.iNext == null?var1:var1 + "...\n" + this.iNext.toString();
      }
   }

   private static class RuleSet {

      private List iRules = new ArrayList();


      RuleSet(ZoneInfoCompiler.Rule var1) {
         this.iRules.add(var1);
      }

      void addRule(ZoneInfoCompiler.Rule var1) {
         if(!var1.iName.equals(((ZoneInfoCompiler.Rule)this.iRules.get(0)).iName)) {
            throw new IllegalArgumentException("Rule name mismatch");
         } else {
            this.iRules.add(var1);
         }
      }

      public void addRecurring(DateTimeZoneBuilder var1, String var2) {
         for(int var3 = 0; var3 < this.iRules.size(); ++var3) {
            ZoneInfoCompiler.Rule var4 = (ZoneInfoCompiler.Rule)this.iRules.get(var3);
            var4.addRecurring(var1, var2);
         }

      }
   }

   private static class Rule {

      public final String iName;
      public final int iFromYear;
      public final int iToYear;
      public final String iType;
      public final ZoneInfoCompiler.DateTimeOfYear iDateTimeOfYear;
      public final int iSaveMillis;
      public final String iLetterS;


      Rule(StringTokenizer var1) {
         this.iName = var1.nextToken().intern();
         this.iFromYear = ZoneInfoCompiler.parseYear(var1.nextToken(), 0);
         this.iToYear = ZoneInfoCompiler.parseYear(var1.nextToken(), this.iFromYear);
         if(this.iToYear < this.iFromYear) {
            throw new IllegalArgumentException();
         } else {
            this.iType = ZoneInfoCompiler.parseOptional(var1.nextToken());
            this.iDateTimeOfYear = new ZoneInfoCompiler.DateTimeOfYear(var1);
            this.iSaveMillis = ZoneInfoCompiler.parseTime(var1.nextToken());
            this.iLetterS = ZoneInfoCompiler.parseOptional(var1.nextToken());
         }
      }

      public void addRecurring(DateTimeZoneBuilder var1, String var2) {
         String var3 = this.formatName(var2);
         this.iDateTimeOfYear.addRecurring(var1, var3, this.iSaveMillis, this.iFromYear, this.iToYear);
      }

      private String formatName(String var1) {
         int var2 = var1.indexOf(47);
         if(var2 > 0) {
            return this.iSaveMillis == 0?var1.substring(0, var2).intern():var1.substring(var2 + 1).intern();
         } else {
            var2 = var1.indexOf("%s");
            if(var2 < 0) {
               return var1;
            } else {
               String var3 = var1.substring(0, var2);
               String var4 = var1.substring(var2 + 2);
               String var5;
               if(this.iLetterS == null) {
                  var5 = var3.concat(var4);
               } else {
                  var5 = var3 + this.iLetterS + var4;
               }

               return var5.intern();
            }
         }
      }

      public String toString() {
         return "[Rule]\nName: " + this.iName + "\n" + "FromYear: " + this.iFromYear + "\n" + "ToYear: " + this.iToYear + "\n" + "Type: " + this.iType + "\n" + this.iDateTimeOfYear + "SaveMillis: " + this.iSaveMillis + "\n" + "LetterS: " + this.iLetterS + "\n";
      }
   }

   static class DateTimeOfYear {

      public final int iMonthOfYear;
      public final int iDayOfMonth;
      public final int iDayOfWeek;
      public final boolean iAdvanceDayOfWeek;
      public final int iMillisOfDay;
      public final char iZoneChar;


      DateTimeOfYear() {
         this.iMonthOfYear = 1;
         this.iDayOfMonth = 1;
         this.iDayOfWeek = 0;
         this.iAdvanceDayOfWeek = false;
         this.iMillisOfDay = 0;
         this.iZoneChar = 119;
      }

      DateTimeOfYear(StringTokenizer var1) {
         int var2 = 1;
         int var3 = 1;
         int var4 = 0;
         int var5 = 0;
         boolean var6 = false;
         char var7 = 119;
         if(var1.hasMoreTokens()) {
            var2 = ZoneInfoCompiler.parseMonth(var1.nextToken());
            if(var1.hasMoreTokens()) {
               String var8 = var1.nextToken();
               if(var8.startsWith("last")) {
                  var3 = -1;
                  var4 = ZoneInfoCompiler.parseDayOfWeek(var8.substring(4));
                  var6 = false;
               } else {
                  try {
                     var3 = Integer.parseInt(var8);
                     var4 = 0;
                     var6 = false;
                  } catch (NumberFormatException var11) {
                     int var10 = var8.indexOf(">=");
                     if(var10 > 0) {
                        var3 = Integer.parseInt(var8.substring(var10 + 2));
                        var4 = ZoneInfoCompiler.parseDayOfWeek(var8.substring(0, var10));
                        var6 = true;
                     } else {
                        var10 = var8.indexOf("<=");
                        if(var10 <= 0) {
                           throw new IllegalArgumentException(var8);
                        }

                        var3 = Integer.parseInt(var8.substring(var10 + 2));
                        var4 = ZoneInfoCompiler.parseDayOfWeek(var8.substring(0, var10));
                        var6 = false;
                     }
                  }
               }

               if(var1.hasMoreTokens()) {
                  var8 = var1.nextToken();
                  var7 = ZoneInfoCompiler.parseZoneChar(var8.charAt(var8.length() - 1));
                  if(var8.equals("24:00")) {
                     if(var2 == 12 && var3 == 31) {
                        var5 = ZoneInfoCompiler.parseTime("23:59:59.999");
                     } else {
                        LocalDate var9 = var3 == -1?(new LocalDate(2001, var2, 1)).plusMonths(1):(new LocalDate(2001, var2, var3)).plusDays(1);
                        var6 = var3 != -1 && var4 != 0;
                        var2 = var9.getMonthOfYear();
                        var3 = var9.getDayOfMonth();
                        if(var4 != 0) {
                           var4 = (var4 - 1 + 1) % 7 + 1;
                        }
                     }
                  } else {
                     var5 = ZoneInfoCompiler.parseTime(var8);
                  }
               }
            }
         }

         this.iMonthOfYear = var2;
         this.iDayOfMonth = var3;
         this.iDayOfWeek = var4;
         this.iAdvanceDayOfWeek = var6;
         this.iMillisOfDay = var5;
         this.iZoneChar = var7;
      }

      public void addRecurring(DateTimeZoneBuilder var1, String var2, int var3, int var4, int var5) {
         var1.addRecurringSavings(var2, var3, var4, var5, this.iZoneChar, this.iMonthOfYear, this.iDayOfMonth, this.iDayOfWeek, this.iAdvanceDayOfWeek, this.iMillisOfDay);
      }

      public void addCutover(DateTimeZoneBuilder var1, int var2) {
         var1.addCutover(var2, this.iZoneChar, this.iMonthOfYear, this.iDayOfMonth, this.iDayOfWeek, this.iAdvanceDayOfWeek, this.iMillisOfDay);
      }

      public String toString() {
         return "MonthOfYear: " + this.iMonthOfYear + "\n" + "DayOfMonth: " + this.iDayOfMonth + "\n" + "DayOfWeek: " + this.iDayOfWeek + "\n" + "AdvanceDayOfWeek: " + this.iAdvanceDayOfWeek + "\n" + "MillisOfDay: " + this.iMillisOfDay + "\n" + "ZoneChar: " + this.iZoneChar + "\n";
      }
   }
}
